from zope.i18n import translate
from plone.formwidget.recurrence import pfr_message, pl_message

PARAMETERS = u"""{{
displayUnactivate: '{display_unactivate}',
displayActivate: '{display_activate}',
edit: '{edit}',
add: '{add}',
refresh: '{refresh}',
title: '{title}',
preview: '{preview}',
addDate: '{add_date}',
recurrenceType: '{recurrence_type}',
dailyInterval1: '{daily_interval_1}',
dailyInterval2: '{daily_interval_2}',
weeklyInterval1: '{weekly_interval_1}',
weeklyInterval2: '{weekly_interval_2}',
weeklyWeekdays: '{weekly_weekdays}',
weeklyWeekdaysHuman: '{weekly_weekdays_human}',
monthlyInterval1: '{monthly_interval_1}',
monthlyInterval2: '{monthly_interval_2}',
monthlyDayOfMonth1: '{monthly_day_of_month_1}',
monthlyDayOfMonth1Human: '{monthly_day_of_month_1_human}',
monthlyDayOfMonth2: '{monthly_day_of_month_2}',
monthlyDayOfMonth3: '{monthly_day_of_month_3}',
monthlyDayOfMonth4: '{monthly_day_of_month_4}',
monthlyWeekdayOfMonth1: '{monthly_weekday_of_month_1}',
monthlyWeekdayOfMonth1Human: '{monthly_weekday_of_month_1_human}',
monthlyWeekdayOfMonth2: '{monthly_weekday_of_month_2}',
monthlyWeekdayOfMonth3: '{monthly_weekday_of_month_3}',
monthlyRepeatOn: '{monthly_repeat_on}',
yearlyInterval1: '{yearly_interval_1}',
yearlyInterval2: '{yearly_interval_2}',
yearlyDayOfMonth1: '{yearly_day_of_month_1}',
yearlyDayOfMonth1Human: '{yearly_day_of_month_1_human}',
yearlyDayOfMonth2: '{yearly_day_of_month_2}',
yearlyDayOfMonth3: '{yearly_day_of_month_3}',
yearlyWeekdayOfMonth1: '{yearly_weekday_of_month_1}',
yearlyWeekdayOfMonth1Human: '{yearly_weekday_of_month_1_human}',
yearlyWeekdayOfMonth2: '{yearly_weekday_of_month_2}',
yearlyWeekdayOfMonth3: '{yearly_weekday_of_month_3}',
yearlyWeekdayOfMonth4: '{yearly_weekday_of_month_4}',
yearlyRepeatOn: '{yearly_repeat_on}',
range: '{range}',
rangeNoEnd: '{range_no_end}',
rangeByOccurrences1: '{range_by_occurrences_1}',
rangeByOccurrences1Human: '{range_by_occurrences_1_human}',
rangeByOccurrences2: '{range_by_occurrences_2}',
rangeByEndDate: '{range_by_end_date}',
rangeByEndDateHuman: '{range_by_end_date_human}',
including: '{including}',
except: '{except}',
cancel: '{cancel}',
save: '{save}',
recurrenceStart: '{recurrence_start}',
additionalDate: '{additional_date}',
include: '{include}',
exclude: '{exclude}',
remove: '{remove}',
orderIndexes: [
    '{order_indexes_first}', '{order_indexes_second}', '{order_indexes_third}',
    '{order_indexes_fourth}', '{order_indexes_last}'],
months: [
    '{month_january}', '{month_february}', '{month_march}', '{month_april}',
    '{month_may}', '{month_june}', '{month_july}', '{month_august}',
    '{month_september}', '{month_october}', '{month_november}', '{month_december}'],
shortMonths: [
    '{month_january_short}', '{month_february_short}', '{month_march_short}',
    '{month_april_short}', '{month_may_short}', '{month_june_short}',
    '{month_july_short}', '{month_august_short}', '{month_september_short}',
    '{month_october_short}', '{month_november_short}', '{month_december_short}'],
weekdays: [
    '{day_sunday}', '{day_monday}', '{day_tuesday}', '{day_wednesday}',
    '{day_thirsday}', '{day_friday}'],
shortWeekdays: [
    '{day_sunday_short}', '{day_monday_short}', '{day_tuesday_short}',
    '{day_wednesday_short}', '{day_thirsday_short}', '{day_friday_short}',
    '{day_saturday_short}'],
longDateFormat: '{long_date_format}',
shortDateFormat: '{short_date_format}',
unsupportedFeatures: '{unsupported_features}',
noTemplateMatch: '{no_template_match}',
multipleDayOfMonth: '{multiple_day_of_month}',
bysetpos: '{bysetpos}',
noRule: '{no_rule}',
noRepeatEvery: '{no_repeat_every}',
noEndDate: '{no_end_date}',
noRepeatOn: '{no_repeat_on}',
pastEndDate: '{past_end_date}',
noEndAfterNOccurrences: '{no_end_after_n_occurrences}',
alreadyAdded: '{already_added}',
rtemplate: {{
    daily: '{template_daily}',
    mondayfriday: '{template_mondayfriday}',
    weekdays: '{template_weekdays}',
    weekly: '{template_weekly}',
    monthly: '{template_monthly}',
    yearly: '{template_yearly}',
    }}
}}"""

# We can create the messages on startup, although translations
# need to be done per request:
messages = {
    'display_unactivate': pfr_message(u'display_unactivate'),
    'display_activate': pfr_message(u'display_activate') ,
    'edit': pfr_message(u'edit'),
    'add': pfr_message(u'add'),
    'refresh': pfr_message(u'refresh'),
    'title': pfr_message(u'title'),
    'preview': pfr_message(u'preview'),
    'add_date': pfr_message(u'add_date'),
    'recurrence_type': pfr_message(u'recurrence_type'),
    'daily_interval_1': pfr_message(u'daily_interval_1'),
    'daily_interval_2': pfr_message(u'daily_interval_2'),
    'weekly_interval_1': pfr_message(u'weekly_interval_1'),
    'weekly_interval_2': pfr_message(u'weekly_interval_2'),
    'weekly_weekdays': pfr_message(u'weekly_weekdays'),
    'weekly_weekdays_human': pfr_message(u'weekly_weekdays_human'),
    'monthly_interval_1': pfr_message(u'monthly_interval_1'),
    'monthly_interval_2': pfr_message(u'monthly_interval_2'),
    'monthly_day_of_month_1': pfr_message(u'monthly_day_of_month_1'),
    'monthly_day_of_month_1_human': pfr_message(u'monthly_day_of_month_1_human'),
    'monthly_day_of_month_2': pfr_message(u'monthly_day_of_month_2'),
    'monthly_day_of_month_3': pfr_message(u'monthly_day_of_month_3'),
    'monthly_day_of_month_4': pfr_message(u'monthly_day_of_month_4'),
    'monthly_weekday_of_month_1': pfr_message(u'monthly_weekday_of_month_1'),
    'monthly_weekday_of_month_1_human': pfr_message(u'monthly_weekday_of_month_1_human'),
    'monthly_weekday_of_month_2': pfr_message(u'monthly_weekday_of_month_2'),
    'monthly_weekday_of_month_3': pfr_message(u'monthly_weekday_of_month_3'),
    'monthly_repeat_on': pfr_message(u'monthly_repeat_on'),
    'yearly_interval_1': pfr_message(u'yearly_interval_1'),
    'yearly_interval_2': pfr_message(u'yearly_interval_2'),
    'yearly_day_of_month_1': pfr_message(u'yearly_day_of_month_1'),
    'yearly_day_of_month_1_human': pfr_message(u'yearly_day_of_month_1_human'),
    'yearly_day_of_month_2': pfr_message(u'yearly_day_of_month_2'),
    'yearly_day_of_month_3': pfr_message(u'yearly_day_of_month_3'),
    'yearly_weekday_of_month_1': pfr_message(u'yearly_weekday_of_month_1'),
    'yearly_weekday_of_month_1_human': pfr_message(u'yearly_weekday_of_month_1_human'),
    'yearly_weekday_of_month_2': pfr_message(u'yearly_weekday_of_month_2'),
    'yearly_weekday_of_month_3': pfr_message(u'yearly_weekday_of_month_3'),
    'yearly_weekday_of_month_4': pfr_message(u'yearly_weekday_of_month_4'),
    'yearly_repeat_on': pfr_message(u'yearly_repeat_on'),
    'range': pfr_message(u'range'),
    'range_no_end': pfr_message(u'range_no_end'),
    'range_by_occurrences_1': pfr_message(u'range_by_occurrences_1'),
    'range_by_occurrences_1_human': pfr_message(u'range_by_occurrences_1_human'),
    'range_by_occurrences_2': pfr_message(u'range_by_occurrences_2'),
    'range_by_end_date': pfr_message(u'range_by_end_date'),
    'range_by_end_date_human': pfr_message(u'range_by_end_date_human'),
    'including': pfr_message(u'including'),
    'except': pfr_message(u'except'),
    'cancel': pfr_message(u'cancel'),
    'save': pfr_message(u'save'),
    'recurrence_start': pfr_message(u'recurrence_start'),
    'additional_date': pfr_message(u'additional_date'),
    'include': pfr_message(u'include'),
    'exclude': pfr_message(u'exclude'),
    'remove': pfr_message(u'remove'),
    'order_indexes_first': pfr_message(u'order_indexes_first'),
    'order_indexes_second': pfr_message(u'order_indexes_second'),
    'order_indexes_third': pfr_message(u'order_indexes_third'),
    'order_indexes_fourth': pfr_message(u'order_indexes_fourth'),
    'order_indexes_last': pfr_message(u'order_indexes_last'),
    'month_january': pl_message(u'month_jan'),
    'month_february': pl_message(u'month_feb'),
    'month_march': pl_message(u'month_mar'),
    'month_april': pl_message(u'month_apr'),
    'month_may': pl_message(u'month_may'),
    'month_june': pl_message(u'month_jun'),
    'month_july': pl_message(u'month_jul'),
    'month_august': pl_message(u'month_aug'),
    'month_september': pl_message(u'month_sep'),
    'month_october': pl_message(u'month_oct'),
    'month_november': pl_message(u'month_nov'),
    'month_december': pl_message(u'month_dec'),
    'month_january_short': pl_message(u'month_jan_abbr'),
    'month_february_short': pl_message(u'month_feb_abbr'),
    'month_march_short': pl_message(u'month_mar_abbr'),
    'month_april_short': pl_message(u'month_apr_abbr'),
    'month_may_short': pl_message(u'month_may_abbr'),
    'month_june_short': pl_message(u'month_jun_abbr'),
    'month_july_short': pl_message(u'month_jul_abbr'),
    'month_august_short': pl_message(u'month_aug_abbr'),
    'month_september_short': pl_message(u'month_sep_abbr'),
    'month_october_short': pl_message(u'month_oct_abbr'),
    'month_november_short': pl_message(u'month_nov_abbr'),
    'month_december_short': pl_message(u'month_dec_abbr'),
    'day_monday': pl_message(u'weekday_mon'),
    'day_tuesday': pl_message(u'weekday_tue'),
    'day_wednesday': pl_message(u'weekday_wed'),
    'day_thirsday': pl_message(u'weekday_thu'),
    'day_friday': pl_message(u'weekday_fri'),
    'day_saturday': pl_message(u'weekday_sat'),
    'day_sunday': pl_message(u'weekday_sun'),
    'day_monday_short': pl_message(u'weekday_mon_abbr'),
    'day_tuesday_short': pl_message(u'weekday_tue_abbr'),
    'day_wednesday_short': pl_message(u'weekday_wed_abbr'),
    'day_thirsday_short': pl_message(u'weekday_thu_abbr'),
    'day_friday_short': pl_message(u'weekday_fri_abbr'),
    'day_saturday_short': pl_message(u'weekday_sat_abbr'),
    'day_sunday_short': pl_message(u'weekday_sun_abbr'),
    'long_date_format': pfr_message(u'long_date_format'),
    'short_date_format': pl_message(u'date_format_short_datepicker'),
    'unsupported_features': pfr_message(u'unsupported_features'),
    'no_template_match': pfr_message(u'no_template_match'),
    'multiple_day_of_month': pfr_message(u'multiple_day_of_month'),
    'bysetpos': pfr_message(u'bysetpos'),
    'no_rule': pfr_message(u'no_rule'),
    'no_repeat_every': pfr_message(u'no_repeat_every'),
    'no_end_date': pfr_message(u'no_end_date'),
    'no_repeat_on': pfr_message(u'no_repeat_on'),
    'past_end_date': pfr_message(u'past_end_date'),
    'no_end_after_n_occurrences': pfr_message(u'no_end_after_n_occurrences'),
    'already_added': pfr_message(u'already_added'),
    'template_daily': pfr_message(u'template_daily'),
    'template_mondayfriday': pfr_message(u'template_mondayfriday'),
    'template_weekdays': pfr_message(u'template_weekdays'),
    'template_weekly': pfr_message(u'template_weekly'),
    'template_monthly': pfr_message(u'template_monthly'),
    'template_yearly': pfr_message(u'template_yearly'),
}

# = translate(messages['display_unactivate'], context=request),
def translations(request):
    xlated = dict([(msg, translate(messages[msg], context=request)) for msg in messages])
    return PARAMETERS.format(**xlated)




from Products.Archetypes.tests.utils import makeContent
from plone.app.testing import TEST_USER_ID
from plone.app.testing import setRoles
from plone.formwidget.recurrence.tests.base import IntegrationTestCase

TESTVALUE = "FREQ=MONTHLY;BYDAY=+3TU;COUNT=5"


class ATWidgetTestCase(IntegrationTestCase):

    def setUp(self):
        self.portal = self.layer['portal']

        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        self.portal.invokeFactory('Folder', 'test-folder')
        setRoles(self.portal, TEST_USER_ID, ['Member'])
        self.folder = self.portal['test-folder']

        self.obj = makeContent(
                self.folder, portal_type='RecurrenceType', id='testobj')
        self.field = self.obj.getField('rec')
        self.widget = self.field.widget

    def test_widget_properties(self):
        widget = self.widget
        self.assertEqual(widget.helper_js,
                ('++resource++jquery.tmpl.js',
                 '++resource++plone.formwidget.recurrence/jquery.recurrenceinput.js',))
        self.assertEqual(widget.helper_css,
                ('++resource++plone.formwidget.recurrence/integration.css',))
        self.assertEqual(widget.macro_edit, 'recurrence_widget')

        self.assertTrue(widget.startField == 'test_start_field')
        self.assertTrue(widget.startYear == 'test_start_year')
        self.assertTrue(widget.startMonth == 'test_start_month')
        self.assertTrue(widget.startDay == 'test_start_day')
        self.assertTrue(widget.first_day == 4)

    def test_widget_process(self):
        self.assertFalse(self.widget.process_form(self.obj, self.field, {}))
        self.assertEqual(
               self.widget.process_form(
                   self.obj, self.field, {'rec': TESTVALUE}),
               (TESTVALUE, {})
        )

    # TODO: A test that renders the widget



from zope.component import adapter
from zope.interface import implementsOnly, implementer, implements
from zope.schema.interfaces import IField
from zope.site import hooks
from zope.traversing.browser import absoluteURL
from z3c.form.widget import Widget, FieldWidget
from z3c.form.browser import widget
from z3c.form.interfaces import IWidget, IFieldWidget, IFormLayer
from plone.formwidget.recurrence.browser.i18n import translations


class IRecurrenceWidget(IWidget):
    """Recurrence widget."""

class RecurrenceWidget(widget.HTMLTextAreaWidget, Widget):
    """Recurrence widget implementation."""
    implementsOnly(IRecurrenceWidget)

    klass = u'recurrence-widget'
    value = u''
    start_field = None

    def update(self):
        super(RecurrenceWidget, self).update()
        widget.addFieldClass(self)

    def site_url(self):
        return absoluteURL(hooks.getSite(), self.request)

    def translation(self):
        return translations(self.request)

    def read_only(self):
        return self.mode == 'display' and 'true' or 'false'

    def get_start_field(self):
        if self.mode == 'display':
            return self.id + '-start'
        return self.form.widgets[self.start_field].js_field

    def get_start_date(self):
        start = self.form.fields[self.start_field].field.get(self.context)
        return start.strftime('%Y-%m-%d %H:%M')

    def first_day(self):
        """ First day of the Week. 0..Sunday, 6..Saturday.
        You can overwrite this default value via widget configuration.
        """
        calendar = self.request.locale.dates.calendars[u'gregorian']
        return calendar.week.get('firstDay', 0)


@adapter(IField, IFormLayer)
@implementer(IFieldWidget)
def RecurrenceFieldWidget(field, request):
    """IFieldWidget factory for RecurrenceWidget."""
    return FieldWidget(field, RecurrenceWidget(request))


# TODO: let's remove this, once an alternative is provided by plone.autoform or
# any other package.

# Use the parameterized widget factory to add widgets with specific parameters.
class ParameterizedFieldWidget(object):
    implements(IFieldWidget)

    def __new__(cls, field, request):
        widget = FieldWidget(field, cls.widget(request))
        for k, v in cls.kw.items():
            setattr(widget, k, v)
        return widget

def ParameterizedWidgetFactory(widget, **kw):
    return type('%sFactory' % widget.__name__,
                (ParameterizedFieldWidget,),
                {'widget': widget, 'kw': kw})



from plone.formwidget.recurrence import tests
from plone.formwidget.recurrence.z3cform import widget
from z3c.form import form, field
from z3c.form.interfaces import IFormLayer
from zope.publisher.browser import TestRequest
import zope.interface
import zope.schema


class ITestForm(zope.interface.Interface):

    recurrence = zope.schema.Text(
        title=u'Recurrence',
        required=True)


class TestEditForm(form.EditForm):

    fields = field.Fields(ITestForm)
    fields['recurrence'].widgetFactory = widget.RecurrenceFieldWidget


class Z3CWidgetTestCase(tests.base.IntegrationTestCase):

    def setUp(self):
        self.portal = self.layer['portal']
        self.portal.portal_quickinstaller.installProduct('plone.formwidget.recurrence')

    # This test doesn't work any longer. We need to set up some real, useful tests.

    def test_widget(self):
        pass
        ## It doens't test very much, since it's all in Javascript...
        #request = TestRequest(skin=IFormLayer)
        #request.LANGUAGE = 'en'
        #form = TestEditForm(self.portal, request)
        #widget = widget.RecurrenceFieldWidget(form.fields['recurrence'].field, request)
        #widget.context = form
        #widget.update()
        #html = widget.render()

        #self.assertTrue('++resource++jquery.tmpl.js' in html)
        #self.assertTrue('++resource++plone.formwidget.recurrence/jquery.recurrenceinput.js' in html)
        #self.assertTrue('++resource++plone.formwidget.recurrence/integration.css' in html)



from dateutil import rrule
from zope.interface import implements
from App.class_init import InitializeClass

from Products.Archetypes.atapi import LinesWidget
from Products.Archetypes.Registry import registerWidget
from Products.validation.interfaces.IValidator import IValidator
from Products.validation import validation


class RecurrenceWidget(LinesWidget):
    _properties = LinesWidget._properties.copy()
    _properties.update({
        'macro_edit': "recurrence_widget",
        'helper_js': ('++resource++jquery.tmpl.js',
                      '++resource++plone.formwidget.recurrence/jquery.recurrenceinput.js'),
        'helper_css': ('++resource++plone.formwidget.recurrence/integration.css',),
        'startField': '',
        'startYear': '',
        'startMonth': '',
        'startDay': '',
        'first_day': '',
        })


InitializeClass(RecurrenceWidget)
registerWidget(RecurrenceWidget,
               title='Recurring Date',
               description=('Renders a recurrence widget to enter all the info '
                            'for recurring dates.'),
               used_for=('plone.app.event.recurrence.RecurrenceField',))


class RecurrenceValidator(object):
    # TODO: tests
    implements(IValidator)

    def __init__(self, name):
        self.name = name

    def __call__(self, value, *args, **kwargs):
        try:
            rrule.rrulestr(value) # TODO: rm dep. on rrule. check with regex
            assert('FREQ' in value) # TODO: check if freq before other
                                    # recurrence parms
        except (ValueError, TypeError, AssertionError):
            return "Validation failed: Please enter valid recurrence data."

        return True
validation.register(RecurrenceValidator('isRecurrence'))



"""Base module for unittesting"""
from plone.app.testing import FunctionalTesting
from plone.app.testing import IntegrationTesting
from plone.app.testing import PLONE_FIXTURE
from plone.app.testing import PloneSandboxLayer
from plone.testing import z2
from zope.interface import Interface
import unittest2 as unittest


class DemoProfile(Interface):
    """Marker interface for our demo GS profile."""


class PloneFormwidgetRecurrenceLayer(PloneSandboxLayer):

    defaultBases = (PLONE_FIXTURE,)

    def setUpZope(self, app, configurationContext):
        """Set up Zope."""
        # Load ZCML
        import plone.formwidget.recurrence
        self.loadZCML(package=plone.formwidget.recurrence)

        import plone.formwidget.recurrence.tests # install AT example types
        self.loadZCML(package=plone.formwidget.recurrence.tests)
        z2.installProduct(app, 'plone.formwidget.recurrence.tests')

    def setUpPloneSite(self, portal):
        """Set up Plone."""
        # install at example types
        self.applyProfile(portal, 'plone.formwidget.recurrence.tests:sample_types')

    def tearDownZope(self, app):
        """Tear down Zope."""
        z2.uninstallProduct(app, 'plone.formwidget.recurrence.tests')


FIXTURE = PloneFormwidgetRecurrenceLayer()
INTEGRATION_TESTING = IntegrationTesting(
    bases=(FIXTURE,), name="PloneFormwidgetRecurrenceLayer:Integration")
FUNCTIONAL_TESTING = FunctionalTesting(
    bases=(FIXTURE,), name="PloneFormwidgetRecurrenceLayer:Functional")


class IntegrationTestCase(unittest.TestCase):
    """Base class for integration tests."""
    layer = INTEGRATION_TESTING


class FunctionalTestCase(unittest.TestCase):
    """Base class for functional tests."""
    layer = FUNCTIONAL_TESTING



from setuptools import setup, find_packages


version = '1.0b7'

setup(name='plone.formwidget.recurrence',
      version=version,
      description="Recurrence widgets for Plone",
      long_description=open("README.rst").read() + "\n" +
                       open("CHANGES.rst").read(),
      # Get more strings from
      # http://pypi.python.org/pypi?%3Aaction=list_classifiers
      classifiers=[
        "Framework :: Plone",
        "Programming Language :: Python",
        ],
      keywords='Plone Event Recurrence Date Time Widget Archetypes z3c.form',
      author='Plone Foundation',
      author_email='plone-developers@lists.sourceforge.net',
      url='https://github.com/plone/plone.formwidget.recurrence',
      license='GPL',
      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['plone', 'plone.formwidget'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'python-dateutil',
          'setuptools',
      ],
      extras_require=dict(
          z3cform=[
              'z3c.form',
          ],
          archetypes=[
              'Products.Archetypes',
          ],
          test=[
              'Products.GenericSetup',
              'z3c.form[test]',
              'plone.app.testing',
              'unittest2'
          ]),
      entry_points="""
      # -*- Entry points: -*-

      [z3c.autoinclude.plugin]
      target = plone
      """,
      )
